==> resh-5.5.0.95-py3-none-any.whl: resh/utils/settings.py <==
import os
import signal
import json
from json import JSONDecodeError

from . import logger
from resh.utils.gateway import gateways_status

HELP = 'help'
VERSION = '--version'
LOG_LEVEL = '-l'
DEBUG = '-d'
JSON = '--json'
VERBOSE = '-v'
FILE = '--file'
INTERACTIVE = 'interactive_mode'
COMPLETION_MODE = '--generate_completions'
INTERFACE = '--gateway'
PROTO = '--proto'

TYPE_DEST = '/boot/rapida/'
TYPE_MASK = 'device-type-'


class Settings(object):
    # Версия протокола R-BUS, определяет формат пакетов. Сейчас, когда есть 2 версии контроллеров. По умолчанию равен
    # версии контролера. Если переключаем интерфейс, меняем на протокол интерфейса. Может быть задан в аргументе PROTO
    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super().__new__(cls, *args, **kwargs)
        return cls._instance

    def __init__(self):
        self.interactive_mode = False
        self.help = False
        self.version = False
        self.debug = False
        self.file = None
        self.completion_mode = False
        self.quiescent = False

        self._device_type = get_device_type()  # Версия контроллера, определяем 1 раз и на весь цикл жизни программы
        self._protocol = self._device_type
        self._interface = 'can0'

        self.net_dev = 'eth0'
        try:
            for _con in json.loads(os.popen('ip -j a').read()):
                if _con.get('link_type') == 'ether':
                    self.net_dev = _con['ifname']
                    break
        except JSONDecodeError:
            pass

    @property
    def device_type(self):
        return self._device_type

    @property
    def interface(self):
        return self._interface

    @interface.setter
    def interface(self, value):
        self._set_interface(value)

    @property
    def protocol(self):
        return self._protocol

    @protocol.setter
    def protocol(self, value):
        try:
            value = int(value)
            if 0 < value < 3:
                self._protocol = value
                return
        except ValueError:
            pass
        print(f'Unexpected protocol value: {value}. Protocol set to 1')
        self._protocol = 1


        value = str(value)
        if value.isdigit():
            value = int(value)
            if 0 < value < 3:
                self.protocol = value
                return
        print(f'Unexpected protocol value: {value}. Protocol set to 1')
        self.protocol = 1


    def parse(self, args: list) -> list:
        if DEBUG in args:
            if not self.debug:
                _index = args.index(DEBUG)
                args.pop(_index)
                if len(args) > _index and '-' not in args[_index]:
                    _value = args.pop(_index)
                    if ':' in _value and str(_value.split(':')[1]).isdigit():
                        _value = _value.split(':')
                        _value[1] = int(_value[1])
                    else:
                        raise BaseException(
                            '"{}" parameter required as "0.0.0.0:0000", but yet as "{}"'.format(DEBUG, _value))
                else:
                    _value = ('10.10.3.134', 54109)
                print('Wait for debug attach from {} ...'.format(':'.join(str(i) for i in _value)))

                def suicide(signum, frame):
                    pid = os.popen('ps | grep resh').read().strip().split()[0]
                    os.system(f'kill -9 {pid}')

                signal.signal(signal.SIGALRM, suicide)
                signal.alarm(3)
                import pydevd_pycharm
                pydevd_pycharm.settrace(_value[0], port=_value[1], stdoutToServer=True, stderrToServer=True)

                signal.alarm(0)

                self.debug = _value
        else:
            self.debug = None

        if HELP in args:
            self.help = True
            args.remove(HELP)
        else:
            self.help = False

        if VERSION in args:
            self.version = True
            args.remove(VERSION)
        else:
            self.version = False

        if JSON in args:
            logger.log.set_json_out(True)
            args.remove(JSON)

        if VERBOSE in args:
            self.quiescent = True
            args.remove(VERBOSE)

        if COMPLETION_MODE in args:
            self.completion_mode = True
            args.remove(COMPLETION_MODE)
        else:
            self.completion_mode = False

        if FILE in args:
            _index = args.index(FILE)
            _q = args.pop(_index)
            if len(args) > _index:
                _value = args.pop(_index)

                if os.path.isfile(_value):
                    self.file = _value
                else:
                    raise BaseException(f'"{_value}" is not a file')
            else:
                raise BaseException(f'"{FILE}" assume file path as string')
        else:
            self.file = None

        if INTERFACE in args:
            _index = args.index(INTERFACE)
            _ = args.pop(_index)
            if len(args) > _index:
                self._set_interface(args.pop(_index))
            else:
                raise BaseException(f'"{INTERFACE}" assume gateway id (1 for "rcan1"")')

        if LOG_LEVEL in args:
            _index = args.index(LOG_LEVEL)
            args.pop(_index)
            if len(args) > _index and '-' not in args[_index]:
                _value = args.pop(_index)   #_value = args[_index]
                if str(_value).isdigit():
                    _value = int(_value)
                else:
                    raise BaseException('"{}" parameter required as <int>, but yet as "{}"'.format(LOG_LEVEL, _value))
            else:
                _value = 1
            logger.log.set_log_level(_value)
        else:
            logger.log.set_log_level(0)

        if PROTO in args:
            _index = args.index(PROTO)
            _proto = args.pop(_index)
            if len(args) > _index:
                _protocol = args.pop(_index)
                self.protocol = _protocol
            else:
                raise BaseException(f'"{PROTO}" assume protocol id (1 for "old", 2 for "rbus2"")')

        self.interactive_mode = args[0] == INTERACTIVE

        return args

    def _set_interface(self, interface):
        interface = str(interface)
        if interface.isdigit():
            _interface = 'can0' if interface == '0' else f'rcan{interface}'
        else:
            _interface = interface
        _protocol = self._device_type
        try:
            if _interface != 'can0':
                _id = _interface[4:]
                if _interface[:4] != 'rcan' or not _id.isdigit():
                    raise ValueError(f'Bad interface format: {_interface}')

                _gateways_status = gateways_status([int(_id)])
                if not _gateways_status:
                    raise ValueError(f'No such gateway interface: {_interface}')

                # Вобщем тут не стал изобретать велосипед. Для виртуального провайдера шины can (rgp) 1ый протокол это 0
                # а второй 1. Когда появится третий, просмотрим как это будет работать, может все переделывать придется
                _protocol = int(_gateways_status[int(_id)]['address']['protocol']) + 1
        except ValueError as e:
            _interface = 'can0'
            print(e)
        finally:
            self._interface = _interface
            self.protocol = _protocol


def get_device_type():
    for f in os.popen('sudo ls %s | grep "%s"' % (TYPE_DEST, TYPE_MASK)).read().strip().split():
        try:
            return int(f.split('device-type-')[1])
        except:
            pass
    return 1


conf = Settings()

==> resh-5.5.0.101-py3-none-any.whl: resh/utils/settings.py <==
import os
import signal
import json
from json import JSONDecodeError

from . import logger
from resh.utils.gateway import gateways_status

HELP = 'help'
VERSION = '--version'
LOG_LEVEL = '-l'
DEBUG = '-d'
JSON = '--json'
VERBOSE = '-v'
FILE = '--file'
INTERACTIVE = 'interactive_mode'
COMPLETION_MODE = '--generate_completions'
INTERFACE = '--gateway'
PROTO = '--proto'

TYPE_DEST = '/boot/rapida/'
TYPE_MASK = 'device-type-'


class Settings(object):
    # Версия протокола R-BUS, определяет формат пакетов. Сейчас, когда есть 2 версии контроллеров. По умолчанию равен
    # версии контролера. Если переключаем интерфейс, меняем на протокол интерфейса. Может быть задан в аргументе PROTO
    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super().__new__(cls, *args, **kwargs)
        return cls._instance

    def __init__(self):
        self.interactive_mode = False
        self.help = False
        self.version = False
        self.debug = False
        self.file = None
        self.completion_mode = False
        self.quiescent = False

        self._device_type = get_device_type()  # Версия контроллера, определяем 1 раз и на весь цикл жизни программы
        self._protocol = self._device_type
        self._interface = 'can0'

        self.net_dev = 'eth0'
        try:
            for _con in json.loads(os.popen('ip -j a').read()):
                if _con.get('link_type') == 'ether':
                    self.net_dev = _con['ifname']
                    break
        except JSONDecodeError:
            pass

    @property
    def device_type(self):
        return self._device_type

    @property
    def interface(self):
        return self._interface

    @interface.setter
    def interface(self, value):
        self._set_interface(value)

    @property
    def protocol(self):
        return self._protocol

    @protocol.setter
    def protocol(self, value):
        try:
            value = int(value)
            if 0 < value < 3:
                self._protocol = value
                return
        except ValueError:
            pass
        print(f'Unexpected protocol value: {value}. Protocol set to 1')
        self._protocol = 1


        value = str(value)
        if value.isdigit():
            value = int(value)
            if 0 < value < 3:
                self.protocol = value
                return
        print(f'Unexpected protocol value: {value}. Protocol set to 1')
        self.protocol = 1


    def parse(self, args: list) -> list:
        if DEBUG in args:
            if not self.debug:
                _index = args.index(DEBUG)
                args.pop(_index)
                if len(args) > _index and '-' not in args[_index]:
                    _value = args.pop(_index)
                    if ':' in _value and str(_value.split(':')[1]).isdigit():
                        _value = _value.split(':')
                        _value[1] = int(_value[1])
                    else:
                        raise BaseException(
                            '"{}" parameter required as "0.0.0.0:0000", but yet as "{}"'.format(DEBUG, _value))
                else:
                    _value = ('10.10.3.134', 54109)
                print('Wait for debug attach from {} ...'.format(':'.join(str(i) for i in _value)))

                def suicide(signum, frame):
                    pid = os.popen('ps | grep resh').read().strip().split()[0]
                    os.system(f'kill -9 {pid}')

                signal.signal(signal.SIGALRM, suicide)
                signal.alarm(3)
                import pydevd_pycharm
                pydevd_pycharm.settrace(_value[0], port=_value[1], stdoutToServer=True, stderrToServer=True)

                signal.alarm(0)

                self.debug = _value
        else:
            self.debug = None

        if HELP in args:
            self.help = True
            args.remove(HELP)
        else:
            self.help = False

        if VERSION in args:
            self.version = True
            args.remove(VERSION)
        else:
            self.version = False

        if JSON in args:
            logger.log.set_json_out(True)
            args.remove(JSON)

        if VERBOSE in args:
            self.quiescent = True
            args.remove(VERBOSE)

        if COMPLETION_MODE in args:
            self.completion_mode = True
            args.remove(COMPLETION_MODE)
        else:
            self.completion_mode = False

        if FILE in args:
            _index = args.index(FILE)
            _q = args.pop(_index)
            if len(args) > _index:
                _value = args.pop(_index)

                if os.path.isfile(_value):
                    self.file = _value
                else:
                    raise BaseException(f'"{_value}" is not a file')
            else:
                raise BaseException(f'"{FILE}" assume file path as string')
        else:
            self.file = None

        if INTERFACE in args:
            _index = args.index(INTERFACE)
            _ = args.pop(_index)
            if len(args) > _index:
                self._set_interface(args.pop(_index))
            else:
                raise BaseException(f'"{INTERFACE}" assume gateway id (1 for "rcan1"")')

        if LOG_LEVEL in args:
            _index = args.index(LOG_LEVEL)
            args.pop(_index)
            if len(args) > _index and '-' not in args[_index]:
                _value = args.pop(_index)   #_value = args[_index]
                if str(_value).isdigit():
                    _value = int(_value)
                else:
                    raise BaseException('"{}" parameter required as <int>, but yet as "{}"'.format(LOG_LEVEL, _value))
            else:
                _value = 1
            logger.log.set_log_level(_value)
        else:
            logger.log.set_log_level(0)

        if PROTO in args:
            _index = args.index(PROTO)
            _proto = args.pop(_index)
            if len(args) > _index:
                _protocol = args.pop(_index)
                self.protocol = _protocol
            else:
                raise BaseException(f'"{PROTO}" assume protocol id (1 for "old", 2 for "rbus2"")')

        self.interactive_mode = args[0] == INTERACTIVE

        return args

    def _set_interface(self, interface):
        interface = str(interface)
        if interface.isdigit():
            _interface = 'can0' if interface == '0' else f'rcan{interface}'
        else:
            _interface = interface
        _protocol = self._device_type
        try:
            if _interface != 'can0':
                _id = _interface[4:]
                if _interface[:4] != 'rcan' or not _id.isdigit():
                    raise ValueError(f'Bad interface format: {_interface}')

                _gateways_status = gateways_status([int(_id)])
                if not _gateways_status:
                    raise ValueError(f'No such gateway interface: {_interface}')

                # Вобщем тут не стал изобретать велосипед. Для виртуального провайдера шины can (rgp) 1ый протокол это 0
                # а второй 1. Когда появится третий, просмотрим как это будет работать, может все переделывать придется
                _protocol = int(_gateways_status[int(_id)]['address']['protocol']) + 1
        except ValueError as e:
            _interface = 'can0'
            print(e)
        finally:
            self._interface = _interface
            self.protocol = _protocol


def get_device_type():
    for f in os.popen('sudo ls %s | grep "%s"' % (TYPE_DEST, TYPE_MASK)).read().strip().split():
        try:
            _type = int(f.split('device-type-')[1])
            # Пока так потому что у нас есть сервера не имеющие своего Can, но с прописанным device-type = 0
            return _type if _type == 2 else 1
        except:
            pass
    return 1


conf = Settings()
